"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Container = void 0;
const XMLStream_1 = require("./xml/XMLStream");
const util_1 = require("util");
const xml2js_1 = require("xml2js");
const Converter_1 = require("./converter/Converter");
const StopsStream_1 = require("./gtfs/StopsStream");
const NaPTAN_1 = require("./reference/NaPTAN");
const TransXChangeStream_1 = require("./transxchange/TransXChangeStream");
const FileStream_1 = require("./converter/FileStream");
const AgencyStream_1 = require("./gtfs/AgencyStream");
const RoutesStream_1 = require("./gtfs/RoutesStream");
const CalendarStream_1 = require("./gtfs/CalendarStream");
const js_joda_1 = require("js-joda");
const TransXChange_1 = require("./transxchange/TransXChange");
const TransXChangeJourneyStream_1 = require("./transxchange/TransXChangeJourneyStream");
const CalendarDatesStream_1 = require("./gtfs/CalendarDatesStream");
const TripsStream_1 = require("./gtfs/TripsStream");
const StopTimesStream_1 = require("./gtfs/StopTimesStream");
const fs = require("fs");
const TransfersStream_1 = require("./gtfs/TransfersStream");
const GetStopData_1 = require("./converter/GetStopData");
const csv_parse_1 = require("csv-parse");
/**
 * Dependency container
 */
class Container {
    async getConverter(refresh, skipStops) {
        if (!skipStops && (!fs.existsSync("/tmp/Stops.csv") || refresh)) {
            console.log("Downloading latest stop data from NAPTAN.");
            await new GetStopData_1.GetStopData().update();
            console.log("Done");
        }
        const files = new FileStream_1.FileStream();
        const xml = new XMLStream_1.XMLStream(this.getParseXML());
        const transxchange = new TransXChangeStream_1.TransXChangeStream();
        const journeyStream = new TransXChangeJourneyStream_1.TransXChangeJourneyStream(this.getBankHolidays());
        const [naptanIndex, locationIndex] = skipStops ? [{}, {}] : await this.getNaPTANIndexes();
        files.pipe(xml).pipe(transxchange).pipe(journeyStream);
        return new Converter_1.Converter(files, {
            "calendar.txt": journeyStream.pipe(new CalendarStream_1.CalendarStream()),
            "calendar_dates.txt": journeyStream.pipe(new CalendarDatesStream_1.CalendarDatesStream()),
            "trips.txt": journeyStream.pipe(new TripsStream_1.TripsStream()),
            "stop_times.txt": journeyStream.pipe(new StopTimesStream_1.StopTimesStream()),
            "agency.txt": transxchange.pipe(new AgencyStream_1.AgencyStream()),
            "routes.txt": transxchange.pipe(new RoutesStream_1.RoutesStream()),
            "transfers.txt": transxchange.pipe(new TransfersStream_1.TransfersStream(naptanIndex, locationIndex)),
            "stops.txt": transxchange.pipe(new StopsStream_1.StopsStream(naptanIndex))
        });
    }
    async getNaPTANIndexes() {
        const naptanFactory = new NaPTAN_1.NaPTANFactory(fs.createReadStream("/tmp/Stops.csv", "utf8").pipe((0, csv_parse_1.parse)()));
        return naptanFactory.getIndexes();
    }
    getParseXML() {
        return (0, util_1.promisify)(xml2js_1.parseString);
    }
    getBankHolidays() {
        const years = ["2020", "2021", "2022", "2023", "2024", "2025", "2026"];
        const dates = {
            ChristmasEve: years.map(year => year + "-12-24"),
            ChristmasDay: years.map(year => year + "-12-25"),
            BoxingDay: years.map(year => year + "-12-26"),
            NewYearsEve: years.map(year => year + "-12-31"),
            NewYearsDay: years.map(year => year + "-01-01"),
            Jan2ndScotland: years.map(year => year + "-01-02"),
            Jan2ndHoliday: [],
            NewYearsDayHoliday: ["2021-01-01", "2022-01-03", "2023-01-02", "2024-01-01", "2025-01-01", "2026-01-01"],
            GoodFriday: ["2020-04-10", "2021-04-02", "2022-04-15", "2023-04-07", "2024-03-29", "2025-04-18", "2026-04-03"],
            EasterMonday: ["2020-04-13", "2021-04-05", "2022-04-18", "2023-04-10", "2024-04-01", "2025-04-21", "2026-04-06"],
            SpringBank: ["2020-05-04", "2021-05-03", "2022-05-02", "2023-05-01", "2024-05-06", "2025-05-05", "2026-05-04"],
            MayDay: ["2020-05-25", "2021-05-31", "2022-05-30", "2023-05-29", "2024-05-27", "2025-05-26", "2026-05-25"],
            LateSummerBankHolidayNotScotland: ["2020-08-31", "2021-08-30", "2022-08-29", "2023-08-28", "2024-08-26", "2025-08-25", "2026-08-31"],
            AugustBankHolidayScotland: ["2020-08-31", "2021-08-30", "2022-08-29", "2023-08-28", "2024-08-26", "2025-08-25", "2026-08-25"],
            ChristmasDayHoliday: ["2020-12-25", "2021-12-27", "2022-12-26", "2023-12-25", "2024-12-25", "2025-12-25", "2026-12-25"],
            BoxingDayHoliday: ["2020-12-28", "2021-12-28", "2022-12-27", "2023-12-26", "2024-12-26", "2025-12-26", "2026-12-28"],
            AllBankHolidays: []
        };
        dates[TransXChange_1.Holiday.AllBankHolidays] = [
            ...dates.NewYearsDayHoliday,
            ...dates.GoodFriday,
            ...dates.EasterMonday,
            ...dates.SpringBank,
            ...dates.MayDay,
            ...dates.LateSummerBankHolidayNotScotland,
            ...dates.ChristmasDayHoliday,
            ...dates.BoxingDayHoliday
        ];
        return Object.entries(dates).reduce((index, [key, dateStrings]) => {
            index[key] = dateStrings.map(date => js_joda_1.LocalDate.parse(date));
            return index;
        }, {});
    }
}
exports.Container = Container;
Container.TMP = "/tmp/transxchange2gtfs/";
