import { ATCOCode } from "../reference/NaPTAN";
import { Duration, LocalDate, LocalTime } from "js-joda";
/**
 * Trimmed down version of the JSON generated by the XML parser
 */
export interface TransXChange {
    StopPoints: StopPoint[];
    JourneySections: JourneyPatternSections;
    Operators: Operators;
    Services: Services;
    VehicleJourneys: VehicleJourney[];
}
/**
 * JourneyPatternSections indexed by id
 */
export declare type JourneyPatternSections = Record<JourneyPatternSectionID, TimingLink[]>;
/**
 * E.g. JPSection-45
 */
export declare type JourneyPatternSectionID = string;
/**
 * AnnotatedStopPointRef from TransXChange feed
 */
export interface StopPoint {
    StopPointRef: ATCOCode;
    CommonName: string;
    LocalityName: string;
    LocalityQualifier: string;
    Location: {
        Latitude: number;
        Longitude: number;
    };
}
/**
 * JourneyPatternTimingLink
 */
export interface TimingLink {
    From: JourneyStop;
    To: JourneyStop;
    RunTime: Duration;
}
/**
 * From/To field inside a JourneyPatternTimingLink
 */
export interface JourneyStop {
    Activity: StopActivity;
    StopPointRef: ATCOCode;
    TimingStatus: string;
    WaitTime?: Duration;
}
/**
 * Types of Activity at a From/To location
 */
export declare enum StopActivity {
    PickUp = "pickUp",
    PickUpAndSetDown = "pickUpAndSetDown",
    SetDown = "setDown",
    Pass = "pass"
}
/**
 * Operators indexed by ID
 */
export declare type Operators = Record<OperatorID, Operator>;
/**
 * Operator ID e.g. OId_MEGA (not to be confused with OperatorCode)
 */
export declare type OperatorID = string;
/**
 * Operator
 */
export interface Operator {
    OperatorCode: string;
    OperatorShortName: string;
    OperatorNameOnLicence: string | undefined;
}
/**
 * e.g. M6_MEGA
 */
export declare type ServiceCode = string;
/**
 * Services indexed by code
 */
export declare type Services = Record<ServiceCode, Service>;
/**
 * Service
 */
export interface Service {
    ServiceCode: ServiceCode;
    Lines: Lines;
    OperatingPeriod: DateRange;
    RegisteredOperatorRef: OperatorID;
    Description: string;
    Mode: Mode;
    ServiceDestination: string;
    ServiceOrigin: string;
    StandardService: JourneyPatterns;
    OperatingProfile: OperatingProfile | undefined;
}
/**
 * JourneyPatterns indexed by pattern ID
 */
export declare type JourneyPatterns = Record<JourneyPatternID, JourneyPattern>;
/**
 * From a JourneyPattern
 */
export interface JourneyPattern {
    Direction: "inbound" | "outbound";
    Sections: JourneyPatternSectionID[];
}
/**
 * JourneyPatternID e.g. JP366
 */
export declare type JourneyPatternID = string;
/**
 * Lines indexed by line ID
 */
export declare type Lines = Record<string, string>;
/**
 * Period of operation
 */
export interface DateRange {
    StartDate: LocalDate;
    EndDate: LocalDate;
}
/**
 * Transport mode
 */
export declare enum Mode {
    Air = "air",
    Bus = "bus",
    Coach = "coach",
    Underground = "underground",
    Ferry = "ferry",
    Train = "train",
    Tram = "tram"
}
/**
 * Vehicle journeys
 */
export interface VehicleJourney {
    OperatingProfile: OperatingProfile;
    ServiceRef: ServiceCode;
    LineRef: string;
    JourneyPatternRef: string;
    DepartureTime: LocalTime;
    VehicleJourneyCode: string;
    OperationalBlockNumber?: string;
}
export interface OperatingProfile {
    RegularDayType: "HolidaysOnly" | DaysOfWeek[];
    SpecialDaysOperation: {
        DaysOfOperation: DateRange[];
        DaysOfNonOperation: DateRange[];
    };
    BankHolidayOperation: {
        DaysOfOperation: Holiday[];
        DaysOfNonOperation: Holiday[];
    };
}
/**
 * Another touch of TransXChange genius.
 */
export declare enum Holiday {
    ChristmasEve = "ChristmasEve",
    ChristmasDay = "ChristmasDay",
    BoxingDay = "BoxingDay",
    NewYearsEve = "NewYearsEve",
    NewYearsDay = "NewYearsDay",
    Jan2ndScotland = "Jan2ndScotland",
    ChristmasDayHoliday = "ChristmasDayHoliday",
    BoxingDayHoliday = "BoxingDayHoliday",
    NewYearsDayHoliday = "NewYearsDayHoliday",
    Jan2ndHoliday = "Jan2ndHoliday",
    LateSummerBankHolidayNotScotland = "LateSummerBankHolidayNotScotland",
    MayDay = "MayDay",
    EasterMonday = "EasterMonday",
    SpringBank = "SpringBank",
    AugustBankHolidayScotland = "AugustBankHolidayScotland",
    GoodFriday = "GoodFriday",
    AllBankHolidays = "AllBankHolidays"
}
/**
 * Integer representation of a boolean
 */
export declare type IntBool = 0 | 1;
/**
 * Days of operation as an array of booleans. 0 = Monday, Sunday = 6
 */
export declare type DaysOfWeek = [IntBool, IntBool, IntBool, IntBool, IntBool, IntBool, IntBool];
