"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.TransXChangeJourneyStream = void 0;
const TransXChange_1 = require("./TransXChange");
const stream_1 = require("stream");
const autobind_decorator_1 = require("autobind-decorator");
const js_joda_1 = require("js-joda");
/**
 * Transforms TransXChange objects into TransXChangeJourneys that are closer to GTFS calendars, calendar dates, trips
 * and stop times.
 */
let TransXChangeJourneyStream = class TransXChangeJourneyStream extends stream_1.Transform {
    constructor(holidays) {
        super({ objectMode: true });
        this.holidays = holidays;
        this.calendars = {};
        this.serviceId = 1;
        this.tripId = 1;
    }
    /**
     * Generate a journey
     */
    _transform(schedule, encoding, callback) {
        for (const vehicle of schedule.VehicleJourneys) {
            try {
                this.processVehicle(schedule, vehicle);
            }
            catch (err) {
                console.log(err);
            }
        }
        callback();
    }
    processVehicle(schedule, vehicle) {
        const service = schedule.Services[vehicle.ServiceRef];
        const journeyPattern = service.StandardService[vehicle.JourneyPatternRef];
        if (!journeyPattern) {
            console.log(`Warning: missing ${vehicle.JourneyPatternRef} on ${vehicle.ServiceRef}`);
            return;
        }
        const sections = journeyPattern.Sections.flatMap(s => schedule.JourneySections[s] || []);
        if (sections.length > 0 && vehicle.OperatingProfile) {
            const calendar = this.getCalendar(vehicle.OperatingProfile, schedule.Services[vehicle.ServiceRef]);
            const stops = this.getStopTimes(sections, vehicle.DepartureTime);
            const route = vehicle.ServiceRef;
            const blockId = vehicle.OperationalBlockNumber;
            const trip = {
                id: this.tripId++,
                shortName: journeyPattern.Direction === "outbound"
                    ? service.ServiceDestination || service.Description
                    : service.ServiceOrigin || service.Description,
                direction: journeyPattern.Direction,
            };
            this.push({ calendar, stops, trip, route, blockId });
        }
    }
    getCalendar(operatingProfile, service) {
        const days = operatingProfile.RegularDayType === "HolidaysOnly"
            ? [0, 0, 0, 0, 0, 0, 0]
            : this.mergeDays(operatingProfile.RegularDayType);
        let startDate = service.OperatingPeriod.StartDate;
        let endDate = service.OperatingPeriod.EndDate;
        let excludes = [];
        let includes = [];
        for (const dates of operatingProfile.SpecialDaysOperation.DaysOfNonOperation) {
            // if the start date of the non-operation is on or before the start of the service date, change the calendar start date
            if (!dates.StartDate.isAfter(startDate)) {
                startDate = dates.EndDate.plusDays(1);
            }
            // if the end date of the non-operation is on or after the end of the service date, change the calendar end date
            else if (!dates.EndDate.isBefore(endDate) || dates.EndDate.year() >= 2037) {
                endDate = dates.StartDate.minusDays(1);
            }
            else if (dates.EndDate.toEpochDay() - dates.StartDate.toEpochDay() < 92) {
                excludes.push(...this.dateRange(dates.StartDate, dates.EndDate, days));
            }
            else {
                console.log("Warning: Ignored extra long break in service", dates, JSON.stringify(service));
            }
        }
        for (const holiday of operatingProfile.BankHolidayOperation.DaysOfNonOperation) {
            excludes.push(...this.getHoliday(holiday, startDate));
        }
        for (const holiday of operatingProfile.BankHolidayOperation.DaysOfOperation) {
            includes.push(...this.getHoliday(holiday, startDate));
        }
        const hash = this.getCalendarHash(days, startDate, endDate, includes, excludes);
        if (!this.calendars[hash]) {
            const id = this.serviceId++;
            this.calendars[hash] = { id, startDate, endDate, days, includes, excludes };
        }
        return this.calendars[hash];
    }
    mergeDays(daysOfOperation) {
        return daysOfOperation.reduce((result, days) => result.map((day, index) => day || days[index]), [0, 0, 0, 0, 0, 0, 0]);
    }
    dateRange(from, to, days, dates = []) {
        if (from.isAfter(to)) {
            return dates;
        }
        else if (days[from.dayOfWeek().value() - 1]) {
            return this.dateRange(from.plusDays(1), to, days, [...dates, from]);
        }
        else {
            return this.dateRange(from.plusDays(1), to, days, dates);
        }
    }
    getHoliday(holiday, startDate) {
        return (this.holidays[holiday] || []).filter(date => date.isAfter(startDate));
    }
    getCalendarHash(days, startDate, endDate, includes, excludes) {
        return [
            days.toString(),
            startDate.toString(),
            endDate.toString(),
            includes.map(d => d.toString()).join(),
            excludes.map(d => d.toString()).join()
        ].join("_");
    }
    getStopTimes(links, departureTime) {
        const stops = [{
                stop: links[0].From.StopPointRef,
                arrivalTime: departureTime.format(js_joda_1.DateTimeFormatter.ofPattern("HH:mm:ss")),
                departureTime: departureTime.format(js_joda_1.DateTimeFormatter.ofPattern("HH:mm:ss")),
                pickup: true,
                dropoff: false,
                exactTime: links[0].From.TimingStatus === "PTP"
            }];
        let lastDepartureTime = js_joda_1.Duration.between(js_joda_1.LocalTime.parse("00:00"), departureTime);
        for (const link of links) {
            const arrivalTime = lastDepartureTime.plusDuration(link.RunTime);
            lastDepartureTime = link.To.WaitTime ? arrivalTime.plusDuration(link.To.WaitTime) : arrivalTime;
            stops.push({
                stop: link.To.StopPointRef,
                arrivalTime: this.getTime(arrivalTime),
                departureTime: this.getTime(lastDepartureTime),
                pickup: link.To.Activity === TransXChange_1.StopActivity.PickUp || link.To.Activity === TransXChange_1.StopActivity.PickUpAndSetDown,
                dropoff: link.To.Activity === TransXChange_1.StopActivity.SetDown || link.To.Activity === TransXChange_1.StopActivity.PickUpAndSetDown,
                exactTime: link.To.TimingStatus === "PTP"
            });
        }
        return stops;
    }
    getTime(time) {
        const hour = time.toHours().toString().padStart(2, "0");
        const minute = (time.toMinutes() % 60).toString().padStart(2, "0");
        return hour + ":" + minute + ":00";
    }
};
TransXChangeJourneyStream = __decorate([
    autobind_decorator_1.default
], TransXChangeJourneyStream);
exports.TransXChangeJourneyStream = TransXChangeJourneyStream;
